@@ -128,7 +128,7 @@ class ASTCPPAttribute(ASTAttribute):
 
     def describe_signature(self, signode: TextElement) -> None:
         signode.append(addnodes.desc_sig_punctuation('[[', '[['))
-        signode.append(nodes.Text(self.arg, self.arg))
+        signode.append(nodes.Text(self.arg))
         signode.append(addnodes.desc_sig_punctuation(']]', ']]'))
 
 
@@ -161,7 +161,7 @@ class ASTGnuAttributeList(ASTAttribute):
 
     def describe_signature(self, signode: TextElement) -> None:
         txt = str(self)
-        signode.append(nodes.Text(txt, txt))
+        signode.append(nodes.Text(txt))
 
 
 class ASTIdAttribute(ASTAttribute):
@@ -174,7 +174,7 @@ class ASTIdAttribute(ASTAttribute):
         return self.id
 
     def describe_signature(self, signode: TextElement) -> None:
-        signode.append(nodes.Text(self.id, self.id))
+        signode.append(nodes.Text(self.id))
 
 
 class ASTParenAttribute(ASTAttribute):
@@ -189,7 +189,31 @@ class ASTParenAttribute(ASTAttribute):
 
     def describe_signature(self, signode: TextElement) -> None:
         txt = str(self)
-        signode.append(nodes.Text(txt, txt))
+        signode.append(nodes.Text(txt))
+
+
+class ASTAttributeList(ASTBaseBase):
+    def __init__(self, attrs: List[ASTAttribute]) -> None:
+        self.attrs = attrs
+
+    def __len__(self) -> int:
+        return len(self.attrs)
+
+    def __add__(self, other: "ASTAttributeList") -> "ASTAttributeList":
+        return ASTAttributeList(self.attrs + other.attrs)
+
+    def _stringify(self, transform: StringifyTransform) -> str:
+        return ' '.join(transform(attr) for attr in self.attrs)
+
+    def describe_signature(self, signode: TextElement) -> None:
+        if len(self.attrs) == 0:
+            return
+        self.attrs[0].describe_signature(signode)
+        if len(self.attrs) == 1:
+            return
+        for attr in self.attrs[1:]:
+            signode.append(addnodes.desc_sig_space())
+            attr.describe_signature(signode)
 
 
 ################################################################################
@@ -423,5 +447,14 @@ class BaseParser:
 
         return None
 
+    def _parse_attribute_list(self) -> ASTAttributeList:
+        res = []
+        while True:
+            attr = self._parse_attribute()
+            if attr is None:
+                break
+            res.append(attr)
+        return ASTAttributeList(res)
+
     def _parse_paren_expression_list(self) -> ASTBaseParenExprList:
         raise NotImplementedError
